unit mlha_f;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  GL, GLU, ExtCtrls, StdCtrls, Menus;

type
  TForm1 = class(TForm)
    Timer: TTimer;
    Edit1: TEdit;
    MainMenu1: TMainMenu;
    Pomoc1: TMenuItem;
    procedure FormCreate(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure TimerTimer(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure Pomoc1Click(Sender: TObject);

  private
    DC: HDC;
    hrc: HGLRC;
    Palette: HPALETTE;

    procedure Ctverec;
    procedure Svetlo;
    procedure SetDCPixelFormat;
    procedure Mlha;

  protected
    procedure Kreslit(var Msg: TWMPaint); message WM_PAINT;
    procedure NovaBarevnaPaleta(var Msg: TWMQueryNewPalette); message WM_QUERYNEWPALETTE;
    procedure ZmenaBarevnePalety(var Msg: TWMPaletteChanged); message WM_PALETTECHANGED;

  public
    X,Y,Z: float;
    UhelX,UhelY,UhelZ,
    Uhel2X,Uhel2Y,Uhel2Z: integer;
  end;

var
  Form1: TForm1;

implementation


{$R *.DFM}


procedure TForm1.SetDCPixelFormat;      //Nastaveni pixelu
var
  hHeap: THandle;
  Barvy, i: Integer;
  Paleta: PLogPalette;
  Cervena, Zelena, Modra: Byte;
  nPixelFormat: Integer;
  pfd: TPixelFormatDescriptor;

begin
  FillChar(pfd, SizeOf(pfd), 0);

  with pfd do begin
    nSize     := sizeof(pfd);                               // Velikost struktury pixelu
    nVersion  := 1;                                         // slo verze
    dwFlags   := PFD_DRAW_TO_WINDOW or
                 PFD_SUPPORT_OPENGL or
                 PFD_DOUBLEBUFFER;                          // struktura pixelu
    iPixelType:= PFD_TYPE_RGBA;                             // RGBA pixelov hodnoty
    cColorBits:= 24;                                        // 24-bit barva
    cDepthBits:= 32;                                        // 32-bit depth buffer
    iLayerType:= PFD_MAIN_PLANE;                            // typ vrstvy
  end;

  nPixelFormat := ChoosePixelFormat(DC, @pfd);
  SetPixelFormat(DC, nPixelFormat, @pfd);

  DescribePixelFormat(DC, nPixelFormat, sizeof(TPixelFormatDescriptor), pfd);

  if ((pfd.dwFlags and PFD_NEED_PALETTE) <> 0) then begin
    Barvy   := 1 shl pfd.cColorBits;
    hHeap     := GetProcessHeap;
    Paleta := HeapAlloc(hHeap, 0, sizeof(TLogPalette) + (Barvy * sizeof(TPaletteEntry)));

    Paleta^.palVersion := $300;
    Paleta^.palNumEntries := Barvy;

    Cervena   := (1 shl pfd.cRedBits) - 1;
    Zelena := (1 shl pfd.cGreenBits) - 1;
    Modra  := (1 shl pfd.cBlueBits) - 1;

    for i := 0 to Barvy - 1 do begin
      Paleta^.palPalEntry[i].peRed   := (((i shr pfd.cRedShift)   and Cervena)   * 255) DIV Cervena;
      Paleta^.palPalEntry[i].peGreen := (((i shr pfd.cGreenShift) and Zelena) * 255) DIV Zelena;
      Paleta^.palPalEntry[i].peBlue  := (((i shr pfd.cBlueShift)  and Modra)  * 255) DIV Modra;
      Paleta^.palPalEntry[i].peFlags := 0;
    end;

    Palette := CreatePalette(Paleta^);
    HeapFree(hHeap, 0, Paleta);

    if (Palette <> 0) then begin
      SelectPalette(DC, Palette, False);
      RealizePalette(DC);
    end;
  end;
end;

procedure TForm1.Svetlo;
const
  glfLightAmbient : Array[0..3] of GLfloat = (0.1, 0.1, 0.1, 1.0);
  glfLightDiffuse : Array[0..3] of GLfloat = (0.7, 0.7, 0.7, 1.0);
  glfLightSpecular: Array[0..3] of GLfloat = (0.0, 0.0, 0.0, 1.0);
  glfLightShininess: GLfloat = 40;
  light_position: Array[0..3] of GLfloat = (100.0,100.0,100.0,1.0);
  glfLightEmission: Array[0..3] of GLfloat = (0.0,0.0,1.0,1.0);

begin
  glEnable(GL_DEPTH_TEST);

  glLightfv(GL_LIGHT0, GL_AMBIENT, @glfLightAmbient);
  glLightfv(GL_LIGHT0, GL_DIFFUSE, @glfLightDiffuse);
  glLightfv(GL_LIGHT0, GL_SPECULAR, @glfLightSpecular);
  glLightfv(GL_LIGHT0, GL_SHININESS, @glfLightShininess);
  glLightfv(GL_LIGHT0, GL_EMISSION, @glfLightEmission);


  glEnable(GL_LIGHTING);
  glEnable(GL_LIGHT0);

end;

procedure TForm1.Mlha;
var
  b,g,r: glfloat;   //barva mlhy
begin
  glEnable(GL_FOG);                       //zapne mlhu
  glFogi(GL_FOG_MODE, GL_EXP);         //vlastnost mlhy
  glFogf(GL_FOG_DENSITY, 0.5);         //intenzita mlhy
  b := 0.7;                              //barva mlhy
  g := 0.7;
  r := 0.7;
  glFogfv (GL_FOG_COLOR, @b);
  glFogfv (GL_FOG_COLOR, @g);
  glFogfv (GL_FOG_COLOR, @r);
  glFogf (GL_FOG_START, 1.0);             //bod, od kterho se zan it mlha
  glFogf (GL_FOG_END, 4.0);               //bod, od kterho je mlha neprhledn

  Edit1.Text := FloatToStr(r); // nevm pro, ale bez tohoto dku nefunguj barvy
end;

procedure TForm1.Ctverec;
var
  b,g,r: glfloat;      //barva objekt
begin
  glClear(GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT);

  //objekt
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity;

  glRotatef(UhelX, 1, 0, 0);			//rotace kamery  X
  glRotatef(UhelY, 0, 1, 0);			//rotace kamery Y
  glRotatef(UhelZ, 0, 0, 1);			//rotace kamery Z
  glTranslatef(X,Y,Z);				//pozice kamery

  b := 0.0;
  g := 0.5;
  r := 1.0;

  glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, @b);
  glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, @g);
  glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, @r);
  glMaterialfv(GL_FRONT, GL_SPECULAR, @b);
  glMaterialfv(GL_FRONT, GL_SPECULAR, @g);
  glMaterialfv(GL_FRONT, GL_SPECULAR, @r);
  glMaterialfv(GL_FRONT, GL_SHININESS, @b);
  glMaterialfv(GL_FRONT, GL_SHININESS, @g);
  glMaterialfv(GL_FRONT, GL_SHININESS, @r);
  glBegin(GL_POLYGON);
    glNormal3f(0.0, 0.0, 1.0);
      glVertex3f(-3.5,-0.5,0.5);
      glVertex3f(-3.5, 0.5,0.5);
      glVertex3f( 3.5, 0.5,0.5);
      glVertex3f( 3.5,-0.5,0.5);
  glEnd;
  glBegin(GL_POLYGON);
    glNormal3f(-1.0, 0.0, 0.0);
      glVertex3f(3.5,-0.5, 6.5);
      glVertex3f(3.5, 0.5, 6.5);
      glVertex3f(3.5, 0.5,0.5);
      glVertex3f(3.5,-0.5,0.5);
  glEnd;
  glBegin(GL_POLYGON);
    glNormal3f(1.0, 0.0, 0.0);
      glVertex3f(-3.5,-0.5, 6.5);
      glVertex3f(-3.5,-0.5,0.5);
      glVertex3f(-3.5, 0.5,0.5);
      glVertex3f(-3.5, 0.5, 6.5);
  glEnd;
  glBegin(GL_POLYGON);
    glNormal3f(0.0, 0.0,-1.0);
      glVertex3f(-3.5,-0.5,6.5);
      glVertex3f( 3.5,-0.5,6.5);
      glVertex3f( 3.5, 0.5,6.5);
      glVertex3f(-3.5, 0.5,6.5);
  glEnd;
  glBegin(GL_POLYGON);
    glNormal3f(0.0, -1.0, 0.0);
      glVertex3f(-3.5, 0.5,0.5);
      glVertex3f(-3.5, 0.5, 6.5);
      glVertex3f( 3.5, 0.5, 6.5);
      glVertex3f( 3.5, 0.5,0.5);
  glEnd;
  glBegin(GL_POLYGON);
    glNormal3f(0.0, 1.0, 0.0);
      glVertex3f(-3.5,-0.5,0.5);
      glVertex3f( 3.5,-0.5,0.5);
      glVertex3f( 3.5,-0.5, 6.5);
      glVertex3f(-3.5,-0.5, 6.5);
  glEnd;


  Edit1.Text := FloatToStr(r); // nevm pro, ale bez tohoto dku nefunguj barvy

  SwapBuffers(DC);
end;


procedure TForm1.FormCreate(Sender: TObject);
begin
  UhelX := 0;
  UhelY := 180;
  UhelZ := 0;
  Uhel2X := 90;
  Uhel2Y := 45;
  Uhel2Z := 45;
  X := 2; Y := 0; Z := -2;
  DC := GetDC(Handle);
  SetDCPixelFormat;
  hrc := wglCreateContext(DC);
  wglMakeCurrent(DC, hrc);
  Svetlo;
  Mlha;
  Timer.Enabled := True;
end;

procedure TForm1.FormResize(Sender: TObject);
var
  PomerStran : GLdouble;

begin
  PomerStran := Width / Height;

  glMatrixMode(GL_PROJECTION);
  glLoadIdentity;
  gluPerspective(30.0, PomerStran, 0.1, 400);
  glViewport(0, 0, Width ,Height);
  InvalidateRect(Handle, nil, False);
end;

procedure TForm1.Kreslit(var Msg: TWMPaint);
var
  ps : TPaintStruct;

begin
  BeginPaint(Handle, ps);
  Ctverec;
  EndPaint(Handle, ps);
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  Timer.Enabled := False;
  wglMakeCurrent(0, 0);
  wglDeleteContext(hrc);
  ReleaseDC(Handle, DC);

  if (Palette <> 0) then
    DeleteObject(Palette);
end;

procedure TForm1.NovaBarevnaPaleta(var Msg : TWMQueryNewPalette);
begin
  if (Palette <> 0) then begin
    Msg.Result := RealizePalette(DC);

    if (Msg.Result <> GDI_ERROR) then
      InvalidateRect(Handle, nil, False);
  end;
end;

procedure TForm1.ZmenaBarevnePalety(var Msg : TWMPaletteChanged);
begin
  if ((Palette <> 0) and (THandle(TMessage(Msg).wParam) <> Handle)) then begin
    if (RealizePalette(DC) <> GDI_ERROR) then
      UpdateColors(DC);

    Msg.Result := 0;
  end;
end;

procedure TForm1.TimerTimer(Sender: TObject);
begin
  InvalidateRect(Handle, nil, False);
  Uhel2X := Uhel2X + 1;     //rotace objektu po souadnici X
  Uhel2Y := Uhel2Y - 2;     //rotace objektu po souadnici Y
  Uhel2Z := Uhel2Z + 3;     //rotace objektu po souadnici Z

  if Uhel2X >= 360 then
    Uhel2X := 0;
  if Uhel2X <= -1 then
    Uhel2X := 359;
  if Uhel2Y >= 360 then
    Uhel2Y := 0;
  if Uhel2Y <= -1 then
    Uhel2Y := 359;
  if Uhel2Z >= 360 then
    Uhel2Z := 0;
  if Uhel2Z <= -1 then
    Uhel2Z := 359;



end;


procedure TForm1.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  case Key of
    VK_UP: begin  //ipka nahoru
        Z := Z + ((-sin((UhelY - 90) * (Pi/180))) * 0.1);
        X := X - ((sin((UhelY) * (Pi/180))) * 0.1);
    end;
  end;
  case Key of
    VK_DOWN: begin //ipka dolu
      Z := Z - ((-sin((UhelY - 90) * (Pi/180))) * 0.1);
      X := X + ((sin((UhelY) * (Pi/180))) * 0.1);
    end;
  end;
  case Key of
    VK_RIGHT: begin //prav ipka
        UhelY := UhelY + 5;
    end;
  end;
  case Key of
    VK_LEFT: begin //lev ipka
        UhelY := UhelY - 5;
    end;
  end;
  case Key of   //klvesa END
    VK_END: begin
      UhelX := UhelX + 5;
  end;
  end;
  case Key of  //klvesa HOME
    VK_HOME: begin
      UhelX := UhelX - 5;
  end;
  end;
end;

procedure TForm1.Pomoc1Click(Sender: TObject);
begin
  ShowMessage('Pohyb pomoc ipek' + chr(13) + 'Pohled nahoru = Home' + chr(13) + 'Pohled dolu = End');
end;

end.

